#!/usr/bin/perl -w 
################# check_snmp_printer_page #################
my $Version='1.0';
# Date : Mai 16 2017
# Author  : Victor Marin
# Help : esia-sa.com
###########################################################

use strict;
use Fcntl;
use Net::SNMP;
use Getopt::Long;
use Encode qw(decode encode);
use DateTime::Format::Strptime;

my $PROGNAME='check_snmp_printer_page.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=$ERRORS{OK};
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $sn;
my $snmp_version = 1;
my $warning = 20;
my $critical = 30;
my $time = "month";
my $ipgesa;
my $nbr_pg_total;
my $nbr_pg_black;
my $nbr_pg_color;
my $nbr_pg_total_inFile;
my $nbr_pg_black_inFile;
my $nbr_pg_color_inFile;
my $date_inFile;
my $today = DateTime->now(time_zone => 'Europe/Paris');

sub print_help() {
	printf "check SNMP printer.\n";
	printf "\tcheck system, \n";
	printf "\tcheck disk status\n";
	printf "\tcheck volume raid status\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning number of impressions (default: 50)\n";
	print " -c                  critical number of impressions (default: 70)\n";
	print " -t					Interval time (default: 1 month)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print " -S (--cherck_nrpe)	usage cherck_nrpe (number sn)\n";
	print " -i (--ip gesa)      Ip gesa\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"S=i"   => \$sn,   "sn=i"  => \$sn,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"t=s" => \$time, "time=s"=> \$time,
	"i=s" => \$ipgesa, "ipgesa=s" => \$ipgesa,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(defined($sn))
{
	open(EXEC, '-|', "/usr/local/esia/plugins/check_nrpe -S $sn -H 0 -c get_printer_stats -a $hostname $community") or die ("Can't exec with nrpe: $!\n");
}
else
{
	open(EXEC, '-|', "/usr/local/esia/plugins/get_printer_stats.pl -H $hostname -C $community") or die ("Can't exec without nrpe: $!\n");
}

while(my $line = <EXEC>) 
{
    chomp $line;
	if (scalar(grep(/page total/, $line)))
	{
		($nbr_pg_total)= ($line =~ /\s*(\d+)\s+page total/);
	}
	if (scalar(grep(/page color/, $line)))
	{
		($nbr_pg_color)= ($line =~ /\s*(\d+)\s+page color/);
	}
	if (scalar(grep(/page black/, $line)))
	{
		($nbr_pg_black)= ($line =~ /\s*(\d+)\s+page black/);
	}
}
close(EXEC);

if(defined($nbr_pg_total) or defined($nbr_pg_color) or defined($nbr_pg_black))
{
	my $folder = "/var/opt/esia_check_snmp_printer_page";
	if( -d "$folder" ) 
	{
	  # le sous-répertoire existe
	}
	else 
	{
		mkdir ($folder,0755) or die ("Error creation folder\n");
	}

	chdir ("$folder");
	if(defined($sn))
	{
		my $file = "$sn-$hostname.txt";
		if( -e "$file" )
		{ 
			open (FICHIER, "+<$file") or die ("You can't read file ");
		}
		else
		{
			open (FICHIER, "+>$file") or die ("You can't create file ");
		}
	}
	else
	{
		if( -e "$hostname.txt" )
		{ 
			open (FICHIER, "+<$hostname.txt") or die ("You can't read file ");
		}
		else
		{
			open (FICHIER, "+>$hostname.txt") or die ("You can't create file ");
		}
	}

	while(defined(my $line = <FICHIER>))
	{
		chomp $line;
		if (scalar(grep(/pgtotal/, $line)))
		{
			($nbr_pg_total_inFile)= ($line =~ /pgtotal=(\d+)/);
		}
		if (scalar(grep(/pgcolor/, $line)))
		{
			($nbr_pg_color_inFile)= ($line =~ /pgcolor=(\d+)/);
		}
		if (scalar(grep(/pgblack/, $line)))
		{
			($nbr_pg_black_inFile)= ($line =~ /pgblack=(\d+)/);
		}
		if (scalar(grep(/date/, $line)))
		{
			($date_inFile)= ($line =~ /date=(.*)/);
			my($y,$M,$d,$h,$m,$s)= ($date_inFile =~	/(\d+)-(\d+)-(\d+).(\d+):(\d+):(\d+)/);
			$date_inFile = DateTime->new(year      => $y,
								 		 month     => $M,
								   		 day       => $d,
										 hour      => $h,
										 minute    => $m,
										 second    => $s);
		}
	}

	if(!defined($date_inFile))
	{
		$date_inFile = $today;
	}

	my $dt =  $date_inFile->strftime('%d/%m/%Y %H:%M:%S');
	if(defined($time))
	{
		if($time eq "year")
		{$date_inFile->add(years => 1);}
		elsif($time eq "month")
		{$date_inFile->add(months => 1);}
		elsif($time eq "week")
		{$date_inFile->add(weeks => 1);}
		elsif($time eq "day")
		{$date_inFile->add(days => 1);}
	}
	else
	{
		$date_inFile->add(months => 1);
	}
	my $cmp = DateTime->compare($today,$date_inFile); #  it returns -1 if $dt1 < $dt2, 0 if $dt1 == $dt2, 1 if $dt1 > $dt2

	if($cmp==1 || $cmp == 0) # Mettre à jour fichier
	{
		seek(FICHIER, 0, 0);
		if(defined($nbr_pg_total))
		{
			print FICHIER "pgtotal=$nbr_pg_total\n";
			$nbr_pg_total = 0;
		}
	
		if(defined($nbr_pg_color))
		{
			print FICHIER "pgcolor=$nbr_pg_color\n";
			$nbr_pg_color = 0;
		}
	
		if(defined($nbr_pg_black))
		{
			print FICHIER "pgblack=$nbr_pg_black\n";
			$nbr_pg_black = 0;
		}
		print FICHIER "date=$today\n";
	}
	else # Faire la soustraction
	{
		if(defined($nbr_pg_total))
		{
			$nbr_pg_total = $nbr_pg_total-$nbr_pg_total_inFile;		
		}
	
		if(defined($nbr_pg_color))
		{
			$nbr_pg_color = $nbr_pg_color-$nbr_pg_color_inFile;				
		}
	
		if(defined($nbr_pg_black))
		{
			$nbr_pg_black = $nbr_pg_black-$nbr_pg_black_inFile;		
		}
	}
	close (FICHIER);

	if($nbr_pg_total < $warning)
	{
		if(defined($nbr_pg_color))
		{
			if(defined($nbr_pg_black))
			{
				print("OK : printed since $dt \n$nbr_pg_total pages \n$nbr_pg_black black pages \n$nbr_pg_color color pages |nbrpg=$nbr_pg_total,nbrblack=$nbr_pg_black,nbrcolor=$nbr_pg_color,warning=$warning,critical=$critical\n");
			}
			else
			{
				$nbr_pg_black=0;
				print("OK : printed since $dt \n$nbr_pg_total pages \n$nbr_pg_color color pages |nbrpg=$nbr_pg_total,nbrblack=$nbr_pg_black,nbrcolor=$nbr_pg_color,warning=$warning,critical=$critical\n");
			}
		}
		else
		{
			$nbr_pg_color = 0;
			if(defined($nbr_pg_black))
			{
				print("OK : printed since $dt \n$nbr_pg_total pages \n$nbr_pg_black black pages |nbrpg=$nbr_pg_total,nbrblack=$nbr_pg_black,nbrcolor=$nbr_pg_color,warning=$warning,critical=$critical\n");
			}
			else
			{
				$nbr_pg_color=0;
				print("OK : printed since $dt \n$nbr_pg_total pages |nbrpg=$nbr_pg_total,nbrblack=$nbr_pg_black,nbrcolor=$nbr_pg_color,warning=$warning,critical=$critical\n");
			}
		}
		exit $ERRORS{'OK'};	
	}
	else
	{
		if($nbr_pg_total > $critical)
		{
			if(defined($nbr_pg_color))
			{
				if(defined($nbr_pg_black))
				{
					print("CRITICAL : printed since $dt \n$nbr_pg_total pages \n$nbr_pg_black black pages \n$nbr_pg_color color pages (critical=$critical) |nbrpg=$nbr_pg_total,nbrblack=$nbr_pg_black,nbrcolor=$nbr_pg_color,warning=$warning,critical=$critical\n");
				}
				else
				{
					$nbr_pg_black=0;
					print("CRITICAL : printed since $dt \n$nbr_pg_total pages \n$nbr_pg_color color pages (critical=$critical) |nbrpg=$nbr_pg_total,nbrblack=$nbr_pg_black,nbrcolor=$nbr_pg_color,warning=$warning,critical=$critical\n");
				}
			}
			else
			{
				$nbr_pg_color = 0;
				if(defined($nbr_pg_black))
				{
					print("CRITICAL : printed since $dt \n$nbr_pg_total pages \n$nbr_pg_black black pages (critical=$critical) |nbrpg=$nbr_pg_total,nbrblack=$nbr_pg_black,nbrcolor=$nbr_pg_color,warning=$warning,critical=$critical\n");
				}
				else
				{
					$nbr_pg_color=0;
					print("CRITICAL : printed since $dt \n$nbr_pg_total pages (critical=$critical) |nbrpg=$nbr_pg_total,nbrblack=$nbr_pg_black,nbrcolor=$nbr_pg_color,warning=$warning,critical=$critical\n");
				}
			}
			exit $ERRORS{'CRITICAL'};
		}
		else
		{
			if(defined($nbr_pg_color))
			{
				if(defined($nbr_pg_black))
				{
					print("WARNING : printed since $dt \n$nbr_pg_total pages \n$nbr_pg_black black pages \n$nbr_pg_color color pages  (warning=$warning) |nbrpg=$nbr_pg_total,nbrblack=$nbr_pg_black,nbrcolor=$nbr_pg_color,warning=$warning,critical=$critical\n");
				}
				else
				{
					$nbr_pg_black=0;
					print("WARNING :  printed since $dt \n$nbr_pg_total pages \n$nbr_pg_color color pages (warning=$warning) |nbrpg=$nbr_pg_total,nbrblack=$nbr_pg_black,nbrcolor=$nbr_pg_color,warning=$warning,critical=$critical\n");
				}
			}
			else
			{
				$nbr_pg_color = 0;
				if(defined($nbr_pg_black))
				{
					print("WARNING : printed since $dt \n$nbr_pg_total pages \n$nbr_pg_black black pages (warning=$warning) |nbrpg=$nbr_pg_total,nbrblack=$nbr_pg_black,nbrcolor=$nbr_pg_color,warning=$warning,critical=$critical\n");
				}
				else
				{
					$nbr_pg_color=0;
					print("WARNING : printed since $dt \n$nbr_pg_total pages (warning=$warning) |nbrpg=$nbr_pg_total,nbrblack=$nbr_pg_black,nbrcolor=$nbr_pg_color,warning=$warning,critical=$critical\n");
				}
			}		
			exit $ERRORS{'WARNING'};
		}
	}
}
else
{
	print("UNKNOWN : error get_printer_stats \n");		
	exit $ERRORS{'UNKNOWN'};
}
